#include <string>
#include <vector>
#include <cmath>
#include <map>
#include <algorithm>
#include <set>
#include <iostream>
#include <sstream>
#include <cstdio>
#include <cassert>
#include <utility>

using namespace std;

struct edge {
    int u, v, c;
    edge(int u, int v, int c):u(u), v(v), c(c) {}
};
typedef vector<edge> graph;

const int INF = (1 << 25) - 1;

template <class T> void sortAndEraseDuplicates(vector<T>& a)
{
    sort(a.begin(), a.end());
    a.erase(unique(a.begin(), a.end()), a.end());
}

template <class T> int indexInSortedVector(const vector<T>& a, const T& v)
{
    return (int)(lower_bound(a.begin(), a.end(), v) - a.begin());
}

template <class T> bool inSet(const vector<T>& a, const T& v)
{
    return binary_search(a.begin(), a.end(), v);
}

class Network {
public:

	struct Arc {
		int u, v;
		int c, p;
		Arc(int u, int v, int c): u(u), v(v), c(c), p(0), rev(0) {
		}
		int rev;
	};

	Network(int n);
    Network(const graph& gr, int n);
	void addEdge(int u, int v, int c);
    int getMaxFlow(int s, int t);
    int getGlobalMinCut(vector<int>& resS);
    int getGlobalOddMinCut();
    int getGlobalOneMinCut();
    int getGlobalThreeMinCut();
	vector<int> getMinimumCut(int s, int t);
private:
	Network(const Network&);
	Network& operator=(const Network&);
	int n;
	vector<vector<Arc> > g;

	bool findAugmentingPath(int v, int t, int cap);
	vector<bool> used;
};

Network::Network(int n): n(n) {
	g.resize(n);
}

void Network::addEdge(int u, int v, int c) {
	assert(u != v);
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(c >= 0);
	Arc uv(u, v, c), vu(v, u, c);
	uv.rev = (int)g[v].size();
	vu.rev = (int)g[u].size();
	g[u].push_back(uv);
	g[v].push_back(vu);
}

bool Network::findAugmentingPath(int v, int t, int cap) {
	if (used[v]) {
		return false;
	}
	used[v] = true;
	if (v == t) {
		return true;
	}
	for(size_t i = 0; i < g[v].size(); ++i) {
		Arc& arc = g[v][i];
		if (arc.c - arc.p >= cap) {
			if (findAugmentingPath(arc.v, t, cap)) {
				g[v][i].p += cap;
				g[arc.v][arc.rev].p -= cap;
				return true;
			}
		}
	}
	return false;
}

int Network::getMaxFlow(int s, int t) {
    for(int i = 0; i < n; ++i) {
		for(size_t j = 0; j < g[i].size(); ++j) {
			g[i][j].p = 0;
		}
	}
    int cap = INF;
	used.resize(n);
    fill(used.begin(), used.end(), false);
    int res = 0;
	while (cap > 0) {
		do {
			fill(used.begin(), used.end(), false);
            res += cap;
		} while (findAugmentingPath(s, t, cap));
        res -= cap;
		cap >>= 1;
	}
    return res;
}

int Network::getGlobalMinCut(vector<int>& resS) {
    int s = 0;
    int res = (1 << 30);
    int rt = -1;
    for(int t = 1; t < n; ++t) {
        int cr = getMaxFlow(s, t);
        if (res > cr) {
            res = cr;
            rt = t;
        }
    }
    resS = getMinimumCut(s, rt);
    return res;
}

int Network::getGlobalOddMinCut() {
    int res = (1 << 30);
    for(int s = 0; s < n; ++s) {
        for(int t = 0; t < n; ++t) if (s != t) {
            vector<int> S = getMinimumCut(s, t);
            if (S.size() % 2 == 1) {
                res = min(res, getMaxFlow(s, t));
            }
        }
    }
    return res;
}

int Network::getGlobalOneMinCut() {
    int res = (1 << 30);
    for(int s = 0; s < n; ++s) {
        int cres = 0;
        for(size_t i = 0; i < g[s].size(); ++i) {
            cres += g[s][i].c;
        }
        res = min(res, cres);
    }
    return res;
}

int Network::getGlobalThreeMinCut() {
    int res = (1 << 30);
    for(int q = 0; q < n; ++q) {
        for(int w = 0; w < q; ++w) {
            for(int e = 0; e < w; ++e) {
                int cres = 0;
                int r[3];
                r[0] = q;
                r[1] = w;
                r[2] = e;
                sort(r, r + 3);
                for(int y = 0; y < 3; ++y) {
                    int s = r[y];
                    for(size_t i = 0; i < g[s].size(); ++i) {
                        if (!binary_search(r, r + 3, g[s][i].v)) {                        
                            cres += g[s][i].c;
                        }
                    }
                }
                res = min(res, cres);
            }
        }
    }
    return res;
}

vector<int> Network::getMinimumCut(int s, int t) {
    getMaxFlow(s, t);
	vector<int> result;
	if (findAugmentingPath(s, t, 1)) {
		assert(false);
	}
	for(int i = 0; i < n; ++i) {
		if (used[i]) {
			result.push_back(i);
		}
	}
	return result;
}

Network::Network(const graph& gr, int n): n(n) {
	g.resize(n);
    for(size_t i = 0; i < gr.size(); ++i) {
        addEdge(gr[i].u, gr[i].v, gr[i].c);
    }
}

class WeightedGraph {
public:
	struct Arc {
		int u, v, c;
		Arc(int u, int v, int c): u(u), v(v), c(c) {
		}
	};
	WeightedGraph() {}
	WeightedGraph(int n);
	void addEdge(int u, int v, int c);
	vector<Arc> getOutgoingArcs(int v) const;
private:
	int n;
	vector<vector<Arc> > g;
};

WeightedGraph::WeightedGraph(int n): n(n) {
	assert(n > 0);
	g.resize(n);
}

void WeightedGraph::addEdge(int u, int v, int c) {
	assert(u != v);
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(c >= 0);
	g[u].push_back(Arc(u, v, c));
	g[v].push_back(Arc(v, u, c));
}

vector<WeightedGraph::Arc> WeightedGraph::getOutgoingArcs(int v) const {
	return g[v];
}

class GomoryHu {
public:

	struct Edge {
		int u, v, c;
		bool inTree;

		Edge(int u, int v, int c, bool inTree = false): u(u), v(v), c(c), inTree(inTree) {
		}

	};

	struct Vertex {
		int id;
		bool fictive;
		Vertex(int id, bool fictive = false): id(id), fictive(fictive) {
		}
	};

	GomoryHu(int n);
    GomoryHu(const graph& g, int n);
	~GomoryHu();
	void addEdge(int u, int v, int c);
	WeightedGraph getGomoryHuTree();
private:
	int n;
	vector<vector<Edge> > g;
	vector<Vertex> vs;

	vector<bool> used;
	void findComponent(int v, vector<int>* comp);
	void constructSpecialGraph();
	bool processComponent(vector<int>& comp);
};

GomoryHu::GomoryHu(int n): n(n) {
	assert(n > 0);
	g.resize(n);
	for(int i = 0; i < n; ++i) {
		vs.push_back(Vertex(i));
	}
}

GomoryHu::~GomoryHu() {
}

void GomoryHu::findComponent(int v, vector<int>* comp) {
	if (used[v]) {
		return ;
	}
	used[v] = true;
	comp->push_back(v);
	for(size_t i = 0; i < g[v].size(); ++i) {
		Edge& e = g[v][i];
		if (!e.inTree) {
			findComponent(e.v, comp);
		}
	}
}

void GomoryHu::addEdge(int u, int v, int c) {
	assert(u != v);
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(c >= 0);
	g[u].push_back(Edge(u, v, c));
	g[v].push_back(Edge(v, u, c));
}

bool GomoryHu::processComponent(vector<int>& comp) {
	vector<int> non_fictive;
	for(size_t i = 0; i < comp.size(); ++i) {
		if (!vs[comp[i]].fictive) {
			non_fictive.push_back(comp[i]);
		}
	}
	if (non_fictive.size() < 2) {
		return false;
	}
	sortAndEraseDuplicates(comp);
	int s = indexInSortedVector(comp, non_fictive[0]),
		t = indexInSortedVector(comp, non_fictive[1]);
	Network network((int)comp.size());
	for(size_t i = 0; i < comp.size(); ++i) {
		int from = comp[i];
		for(size_t j = 0; j < g[from].size(); ++j) {
			Edge& e = g[from][j];
			int to = e.v;
			if (inSet(comp, to)) {
				network.addEdge((int)i, indexInSortedVector(comp, to), e.c);
			}
		}
	}

	vector<int> S =	network.getMinimumCut(s, t), T;
	sortAndEraseDuplicates(S);
	for(size_t i = 0; i < S.size(); ++i) {
		S[i] = comp[S[i]];
	}
	for(size_t i = 0; i < comp.size(); ++i) {
		if (!inSet(S, comp[i])) {
			T.push_back(comp[i]);
		}
	}

	int fict_s = n, fict_t = n + 1;
	vs.push_back(Vertex(fict_s, true));
	vs.push_back(Vertex(fict_t, true));
	n += 2;
	g.resize(n);

	int cut = 0;
	for(size_t i = 0; i < comp.size(); ++i) {
		int v = comp[i];
		int fict;
		vector<int>* oppSet;
		if (inSet(S, v)) {
			fict = fict_s;
			oppSet = &T;
		} else {
			fict = fict_t;
			oppSet = &S;
		}
		for(size_t j = 0; j < g[v].size(); ++j) {
			Edge& e = g[v][j];
			if (inSet(*oppSet, e.v)) {
				cut += e.c;
				e.v = fict;
				Edge e_rev(fict, v, e.c);
				g[fict].push_back(e_rev);
			}
		}
	}
	assert(cut % 2 == 0);
	cut /= 2;

	g[fict_s].push_back(Edge(fict_s, fict_t, cut, true));
	g[fict_t].push_back(Edge(fict_t, fict_s, cut, true));
	return true;
}

void GomoryHu::constructSpecialGraph() {
	bool found = true;
   	vector<int> comp_init;
	used.resize(n);
	fill(used.begin(), used.end(), false);
   	findComponent(0, &comp_init);
    assert((int)comp_init.size() == n);
	while (found) {
		found = false;
		used.resize(n);
		fill(used.begin(), used.end(), false);
		for(int v = 0; v < n; ++v) {
			if (!vs[v].fictive) {
				vector<int> comp;
				findComponent(v, &comp);
				if (processComponent(comp)) {
					found = true;
					break;
				}
			}
		}
	}
}

WeightedGraph GomoryHu::getGomoryHuTree() {
	WeightedGraph tree(n);
	constructSpecialGraph();
	used.resize(n);
	fill(used.begin(), used.end(), false);
	vector<int> num_comp(n);
	for(int v = 0; v < n; ++v) {
		if (!vs[v].fictive) {
			vector<int> comp;
			findComponent(v, &comp);
			for(size_t i = 0; i < comp.size(); ++i) {
				num_comp[comp[i]] = v;
			}
		}
	}
	for(int v = 0; v < n; ++v) {
		for(size_t j = 0; j < g[v].size(); ++j) {
			Edge& e = g[v][j];
			if (e.inTree && v < e.v) {
				tree.addEdge(num_comp[v], num_comp[e.v], e.c);
			}
		}
	}
	return tree;
}

GomoryHu::GomoryHu(const graph& gr, int n): n(n) {
	g.resize(n);
    for(int i = 0; i < n; ++i) {
		vs.push_back(Vertex(i));
	}
    for(size_t i = 0; i < gr.size(); ++i) {
        addEdge(gr[i].u, gr[i].v, gr[i].c);
    }
}

class OddCut {
public:
    int findCut(int n, GomoryHu& gomoryHu, vector<int>& res) {
        tree = gomoryHu.getGomoryHuTree();
    	k_subtree = vector<int>(n, -1);
    	parent = vector<int>(n, -1);
    	calcSubtree(0, tree);
    	int ans = INF;
    	WeightedGraph::Arc ans_arc(-1, -1, -1);
    	for(int v = 0; v < n; ++v) {
    		vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    		for(size_t i = 0; i < c.size(); ++i) {
    			WeightedGraph::Arc& arc = c[i];
    			int x = arc.v;
    			int y = v;
    			if (parent[x] != y) {
    				swap(x, y);
    			}
    			if (k_subtree[x] % 2 == 1) {
    				if (ans > arc.c) {
    					ans = arc.c;
    					ans_arc = WeightedGraph::Arc(x, y, arc.c);
    				}
    			}
    		}
    	}
    	vector<int> S;
    	findSubtree(ans_arc.v, ans_arc.u, tree, &S);
        res = S;
        return ans;
    }

private:
    vector<int> k_subtree, parent;

    int calcSubtree(int v, const WeightedGraph& tree) {
    	k_subtree[v] = 1;
    	vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    	for(size_t i = 0; i < c.size(); ++i) {
    		int u = c[i].v;
    		if (k_subtree[u] == -1) {
    			parent[u] = v;
    			k_subtree[v] += calcSubtree(u, tree);
    		}
    	}
    	return k_subtree[v];
    }

    void findSubtree(int v, int p, const WeightedGraph& tree, vector<int>* subtree) {
    	subtree->push_back(v);
    	vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    	for(size_t i = 0; i < c.size(); ++i) {
    		int u = c[i].v;
    		if (u != p) {
    			findSubtree(u, v, tree, subtree);
    		}
    	}
    }

    WeightedGraph tree;
};

int n, m;

void solve(GomoryHu& gomoryHu) {
    OddCut oddCut;
    vector<int> S;
    int ans = oddCut.findCut(n, gomoryHu, S);
	printf("%d\n", ans);
	printf("%d\n", S.size());
	for(size_t i = 0; i < S.size(); ++i) {
        if (i > 0) printf(" ");
		printf("%d", 1 + S[i]);
	}
	printf("\n");
}

void readdata() {
	scanf("%d%d", &n, &m);
    assert(n % 2 == 0);
    assert(1 <= n && n <= 200);
    assert(1 <= m && m <= 2000);
	GomoryHu gomoryHu(n);
	for(int i = 0; i < m; ++i) {
		int x, y, c;
		scanf("%d%d%d", &x, &y, &c);
		assert(x != y);
    	assert(1 <= x && x <= n);
    	assert(1 <= y && y <= n);
        assert(1 <= c && c <= 100000);
        --x; --y;
		gomoryHu.addEdge(x, y, c);
	}
    solve(gomoryHu);
}

int main() {
	readdata();
	return 0;
}
