#include <string>
#include <vector>
#include <cmath>
#include <map>
#include <algorithm>
#include <set>
#include <iostream>
#include <queue>
#include <sstream>
#include <cstdio>
#include <cassert>
#include <cstring>
#include <utility>

using namespace std;

const int INF = (1 << 25) - 1;

template <class T> void sortAndEraseDuplicates(vector<T>& a)
{
    sort(a.begin(), a.end());
    a.erase(unique(a.begin(), a.end()), a.end());
}

template <class T> int indexInSortedVector(const vector<T>& a, const T& v)
{
    return (int)(lower_bound(a.begin(), a.end(), v) - a.begin());
}

template <class T> bool inSet(const vector<T>& a, const T& v)
{
    return binary_search(a.begin(), a.end(), v);
}

//---
class Network {
public:

	struct Arc {

		int u;
		int v;
		int c;
		int f;
		Arc *rev;
		
		Arc(int u, int v, int c): u(u), v(v), c(c), f(0), rev(0) {
		}
	};

	Network(int n);
	~Network();

	void addEdge(int u, int v, int c);
	int getMaximumFlow(int s, int t);
	std::vector<int> getMinimumCut(int s, int t);
	int getMinimumCut();

private:

	static const double RATIO = 2.0;

	Network(const Network&);
	Network& operator=(const Network&);

	int n, m;
	std::vector<Arc*> *edges;

	int *d;
	int *exc;
	int *curArc;
	bool *used;
	
	int work;
	
	std::multimap<int, int> active;
	
	bool isAdmissible(Arc *a);	
	int push(Arc *a);
	void relabel(int v);
	void discharge(int v);
	void dfs(int v);
	void globalRelabeling(int s, int t);
};

Network::Network(int n): n(n) {
	assert(n > 0);
	edges = new std::vector<Arc*>[n];
	d = new int[n];
	exc = new int[n];
	curArc = new int[n];
	used = new bool[n];
	work= 0;
	m = 0;
}

Network::~Network() {
	for (int i = 0; i < n; i++) {
		for (std::vector<Arc*>::iterator it = edges[i].begin(); it != edges[i].end(); it++) {
			delete *it;
		}
	}
	delete[] edges;
	delete[] d;
	delete[] exc;
	delete[] curArc;
	delete[] used;
}

void Network::addEdge(int u, int v, int c) {
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(u != v);
	assert(c >= 0);
	Arc *a = new Arc(u, v, c);
	Arc *b = new Arc(v, u, c);
	a->rev = b;
	b->rev = a;
	edges[u].push_back(a);
	edges[v].push_back(b);
	m += 2;
}

void checkArray(std::vector<int> &a) {
	int old = a.size();
	std::sort(a.begin(), a.end());
	a.erase(std::unique(a.begin(), a.end()), a.end());
	assert(a.size() == old);
}

int Network::push(Arc *a) {
	work++;
	int delta = std::min(a->c - a->f, exc[a->u]);
	exc[a->u] -= delta;
	exc[a->v] += delta;
	a->f += delta;
	a->rev->f -= delta;
	return delta;
}

void Network::relabel(int v) {
	d[v] = 2 * n;
	for (std::vector<Arc*>::iterator it = edges[v].begin(); it != edges[v].end(); it++) {
		work++;
		if ((*it)->f < (*it)->c && d[(*it)->v] + 1 < d[v]) {
			d[v] = d[(*it)->v] + 1;
		}
	}
}

bool Network::isAdmissible(Arc *a) {
	return a->f < a->c && d[a->u] == d[a->v] + 1;
}

void Network::discharge(int v) {
	bool needRelabel = false;
	for (;;) {
		if (isAdmissible(edges[v][curArc[v]])) {
			int delta = push(edges[v][curArc[v]]);
			if (exc[edges[v][curArc[v]]->v] == delta) {
				if (d[edges[v][curArc[v]]->v] < n) {
					active.insert(std::make_pair(d[edges[v][curArc[v]]->v], edges[v][curArc[v]]->v));
				}
			}
		}
		else {
			if (curArc[v] != edges[v].size() - 1) curArc[v]++;
			else {
				curArc[v] = 0;
				needRelabel = true;
				break;
			}
		}
		if (!exc[v] || needRelabel) break;
	}
	if (needRelabel) relabel(v);
}

void Network::globalRelabeling(int s, int t) {
	std::queue<int>	q;
	q.push(t);
	for (int i = 0; i < n; i++) {
		d[i] = n;
	}
	d[t] = 0;
	while (!q.empty()) {
		int v = q.front();
		q.pop();
		for (vector<Arc*>::iterator it = edges[v].begin(); it != edges[v].end(); it++) {
			Arc *a = (*it)->rev;
			if (a->f < a->c && d[v] + 1 < d[a->u]) {
				d[a->u] = d[v] + 1;
				q.push(a->u);
			}
		}
	}
	active.clear();
	for (int i = 0; i < n; i++) {
		if (i == s || i == t) continue;
		if (!exc[i]) continue;
		if (d[i] >= n) continue;
		active.insert(std::make_pair(d[i], i));
	}
}

int Network::getMaximumFlow(int s, int t) {
	assert(0 <= s && s < n);
	assert(0 <= t && t < n);
	assert(s != t);
	for (int i = 0; i < n; i++) {
		d[i] = 0;
	}
	d[s] = n;
	for (int i = 0; i < n; i++) {
		exc[i] = 0;
		for (std::vector<Arc*>::iterator it = edges[i].begin(); it != edges[i].end(); it++) {
			(*it)->f = 0;
		}
	}
	for (std::vector<Arc*>::iterator it = edges[s].begin(); it != edges[s].end(); it++) {
		(*it)->f = (*it)->c;
		(*it)->rev->f = -(*it)->c;
		exc[(*it)->u] -= (*it)->c;
		exc[(*it)->v] += (*it)->c;
	}
	active.clear();
	for (int i = 0; i < n; i++) {
		curArc[i] = 0;
		if (i == s || i == t) continue;
		if (!exc[i]) continue;
		active.insert(std::make_pair(d[i], i));
	}
	while (!active.empty()) {
		std::multimap<int, int>::iterator it = active.end();
		it--;
		int node = it->second;
		assert(d[node] == it->first);
		assert(d[node] < n);
		//printf("%d\n", d[node]);
		active.erase(it);
		if (node == s || node == t) continue;
		discharge(node);
		if (exc[node] > 0) {
			if (d[node] < n) {
				active.insert(std::make_pair(d[node], node));
			}
		}
		if (work > RATIO * m) {
			work = 0;
			globalRelabeling(s, t);
		}
	}
	int res = 0;
	for (std::vector<Arc*>::iterator it = edges[t].begin(); it != edges[t].end(); it++) {
		res += (*it)->rev->f;
	}
	return res;
}

void Network::dfs(int v) {
	if (used[v]) return;
	used[v] = true;
	for (std::vector<Arc*>::iterator it = edges[v].begin(); it != edges[v].end(); it++) {
		Arc *a = (*it)->rev;
		assert(a->v == v);
		if (a->f < a->c) dfs(a->u);
	}
}

std::vector<int> Network::getMinimumCut(int s, int t) {
	getMaximumFlow(s, t);
	memset(used, 0, n * sizeof(bool));
	dfs(t);
	std::vector<int> res;
	for (int i = 0; i < n; i++) {
		if (used[i]) res.push_back(i);
	}
	return res;
}

int Network::getMinimumCut() {
	int res = 1e9;
	for (int i = 1; i < n; i++) {
		res = std::min(res, getMaximumFlow(0, i));
	}
	return res;
}
//---

class WeightedGraph {
public:
	struct Arc {
		int u, v, c;
		Arc(int u, int v, int c): u(u), v(v), c(c) {
		}
	};
	WeightedGraph() {}
	WeightedGraph(int n);
	void addEdge(int u, int v, int c);
	vector<Arc> getOutgoingArcs(int v) const;
private:
	int n;
	vector<vector<Arc> > g;
};

WeightedGraph::WeightedGraph(int n): n(n) {
	assert(n > 0);
	g.resize(n);
}

void WeightedGraph::addEdge(int u, int v, int c) {
	assert(u != v);
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(c >= 0);
	g[u].push_back(Arc(u, v, c));
	g[v].push_back(Arc(v, u, c));
}

vector<WeightedGraph::Arc> WeightedGraph::getOutgoingArcs(int v) const {
	return g[v];
}

class GomoryHu {
public:

	struct Edge {
		int u, v, c;
		bool inTree;

		Edge(int u, int v, int c, bool inTree = false): u(u), v(v), c(c), inTree(inTree) {
		}

	};

	struct Vertex {
		int id;
		bool fictive;
		Vertex(int id, bool fictive = false): id(id), fictive(fictive) {
		}
	};

	GomoryHu(int n);
	~GomoryHu();
	void addEdge(int u, int v, int c);
	WeightedGraph getGomoryHuTree();
private:
	int n;
	vector<vector<Edge> > g;
	vector<Vertex> vs;

	vector<bool> used;
	void findComponent(int v, vector<int>* comp);
	void constructSpecialGraph();
	bool processComponent(vector<int>& comp);
};

GomoryHu::GomoryHu(int n): n(n) {
	assert(n > 0);
	g.resize(n);
	for(int i = 0; i < n; ++i) {
		vs.push_back(Vertex(i));
	}
}

GomoryHu::~GomoryHu() {
}

void GomoryHu::findComponent(int v, vector<int>* comp) {
	if (used[v]) {
		return ;
	}
	used[v] = true;
	comp->push_back(v);
	for(size_t i = 0; i < g[v].size(); ++i) {
		Edge& e = g[v][i];
		if (!e.inTree) {
			findComponent(e.v, comp);
		}
	}
}


void GomoryHu::addEdge(int u, int v, int c) {
	assert(u != v);
	assert(0 <= u && u < n);
	assert(0 <= v && v < n);
	assert(c >= 0);
	g[u].push_back(Edge(u, v, c));
	g[v].push_back(Edge(v, u, c));
}

bool GomoryHu::processComponent(vector<int>& comp) {
	vector<int> non_fictive;
	for(size_t i = 0; i < comp.size(); ++i) {
		if (!vs[comp[i]].fictive) {
			non_fictive.push_back(comp[i]);
		}
	}
	if (non_fictive.size() < 2) {
		return false;
	}
	sortAndEraseDuplicates(comp);
	int s = indexInSortedVector(comp, non_fictive[0]),
		t = indexInSortedVector(comp, non_fictive[1]);
	Network network((int)comp.size());
	for(size_t i = 0; i < comp.size(); ++i) {
		int from = comp[i];
		for(size_t j = 0; j < g[from].size(); ++j) {
			Edge& e = g[from][j];
			int to = e.v;
			if (inSet(comp, to)) {
				network.addEdge((int)i, indexInSortedVector(comp, to), e.c);
			}
		}
	}

	vector<int> S =	network.getMinimumCut(s, t), T;
	sortAndEraseDuplicates(S);
	for(size_t i = 0; i < S.size(); ++i) {
		S[i] = comp[S[i]];
	}
	for(size_t i = 0; i < comp.size(); ++i) {
		if (!inSet(S, comp[i])) {
			T.push_back(comp[i]);
		}
	}

	int fict_s = n, fict_t = n + 1;
	vs.push_back(Vertex(fict_s, true));
	vs.push_back(Vertex(fict_t, true));
	n += 2;
	g.resize(n);

	int cut = 0;
	for(size_t i = 0; i < comp.size(); ++i) {
		int v = comp[i];
		int fict;
		vector<int>* oppSet;
		if (inSet(S, v)) {
			fict = fict_s;
			oppSet = &T;
		} else {
			fict = fict_t;
			oppSet = &S;
		}
		for(size_t j = 0; j < g[v].size(); ++j) {
			Edge& e = g[v][j];
			if (inSet(*oppSet, e.v)) {
				cut += e.c;
				e.v = fict;
				Edge e_rev(fict, v, e.c);
				g[fict].push_back(e_rev);
			}
		}
	}
	assert(cut % 2 == 0);
	cut /= 2;

	g[fict_s].push_back(Edge(fict_s, fict_t, cut, true));
	g[fict_t].push_back(Edge(fict_t, fict_s, cut, true));
	return true;
}

void GomoryHu::constructSpecialGraph() {
	bool found = true;
   	vector<int> comp_init;
	used.resize(n);
	fill(used.begin(), used.end(), false);
   	findComponent(0, &comp_init);
    assert((int)comp_init.size() == n);
	while (found) {
		found = false;
		used.resize(n);
		fill(used.begin(), used.end(), false);
		for(int v = 0; v < n; ++v) {
			if (!vs[v].fictive) {
				vector<int> comp;
				findComponent(v, &comp);
				if (processComponent(comp)) {
					found = true;
					break;
				}
			}
		}
	}
}

WeightedGraph GomoryHu::getGomoryHuTree() {
	WeightedGraph tree(n);
	constructSpecialGraph();
	used.resize(n);
	fill(used.begin(), used.end(), false);
	vector<int> num_comp(n);
	for(int v = 0; v < n; ++v) {
		if (!vs[v].fictive) {
			vector<int> comp;
			findComponent(v, &comp);
			for(size_t i = 0; i < comp.size(); ++i) {
				num_comp[comp[i]] = v;
			}
		}
	}
	for(int v = 0; v < n; ++v) {
		for(size_t j = 0; j < g[v].size(); ++j) {
			Edge& e = g[v][j];
			if (e.inTree && v < e.v) {
				tree.addEdge(num_comp[v], num_comp[e.v], e.c);
			}
		}
	}
	return tree;
}

class OddCut {
public:
    int findCut(int n, GomoryHu& gomoryHu, vector<int>& res) {
        tree = gomoryHu.getGomoryHuTree();
    	k_subtree = vector<int>(n, -1);
    	parent = vector<int>(n, -1);
    	calcSubtree(0, tree);
    	int ans = INF;
    	WeightedGraph::Arc ans_arc(-1, -1, -1);
    	for(int v = 0; v < n; ++v) {
    		vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    		for(size_t i = 0; i < c.size(); ++i) {
    			WeightedGraph::Arc& arc = c[i];
    			int x = arc.v;
    			int y = v;
    			if (parent[x] != y) {
    				swap(x, y);
    			}
    			if (k_subtree[x] % 2 == 1) {
    				if (ans > arc.c) {
    					ans = arc.c;
    					ans_arc = WeightedGraph::Arc(x, y, arc.c);
    				}
    			}
    		}
    	}
    	vector<int> S;
    	findSubtree(ans_arc.v, ans_arc.u, tree, &S);
        res = S;
        return ans;
    }

private:
    vector<int> k_subtree, parent;

    int calcSubtree(int v, const WeightedGraph& tree) {
    	k_subtree[v] = 1;
    	vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    	for(size_t i = 0; i < c.size(); ++i) {
    		int u = c[i].v;
    		if (k_subtree[u] == -1) {
    			parent[u] = v;
    			k_subtree[v] += calcSubtree(u, tree);
    		}
    	}
    	return k_subtree[v];
    }

    void findSubtree(int v, int p, const WeightedGraph& tree, vector<int>* subtree) {
    	subtree->push_back(v);
    	vector<WeightedGraph::Arc> c = tree.getOutgoingArcs(v);
    	for(size_t i = 0; i < c.size(); ++i) {
    		int u = c[i].v;
    		if (u != p) {
    			findSubtree(u, v, tree, subtree);
    		}
    	}
    }

    WeightedGraph tree;
};

int n, m;

void solve(GomoryHu& gomoryHu) {
    OddCut oddCut;
    vector<int> S;
    int ans = oddCut.findCut(n, gomoryHu, S);
	printf("%d\n", ans);
	printf("%d\n", S.size());
	for(size_t i = 0; i < S.size(); ++i) {
        if (i > 0) printf(" ");
		printf("%d", 1 + S[i]);
	}
	printf("\n");
}

void readdata() {
	if (scanf("%d%d", &n, &m) != 2) assert(false);
    assert(n % 2 == 0);
    assert(1 <= n && n <= 200);
    assert(1 <= m && m <= 2000);
	GomoryHu gomoryHu(n);
	Network network(n);	
	for(int i = 0; i < m; ++i) {
		int x, y, c;
		if (scanf("%d%d%d", &x, &y, &c) != 3) assert(false);
		assert(x != y);
    	assert(1 <= x && x <= n);
    	assert(1 <= y && y <= n);
        assert(1 <= c && c <= 100000);
        --x; --y;
		gomoryHu.addEdge(x, y, c);
		network.addEdge(x, y, c);
	}
    solve(gomoryHu);
//	printf("%d\n", network.getMinimumCut());
}

int main() {
	readdata();	
	return 0;
}
