#define _CRT_SECURE_NO_WARNINGS

#include <string>
#include <vector>
#include <cmath>
#include <map>
#include <algorithm>
#include <set>
#include <iostream>
#include <sstream>
#include <cstdio>
#include <cassert>
#include <utility>

using namespace std;

template <class T> int indexInSortedVector(const vector<T>& a, const T& v)
{
    return (int)(lower_bound(a.begin(), a.end(), v) - a.begin());
}

class Automat {
public:
	Automat(const string& pattern);
	int getNumberStates() const;
	int goFromState(int from, char c) const;
	bool isTerminal(int s) const;
	int getInit() const;
	~Automat();
private:
	Automat(const Automat&);
	Automat& operator= (const Automat&);
	int n;
	int len;
	string pattern;
	vector<int> states;
	vector<vector<int> > p;
	int s;

	void findMasks(int cmsk, set<int>& msks);
	int appendStars(int msk) const;
	int nextMask(int cmsk, char c);
};

int Automat::getInit() const {
	return s;
}

int Automat::nextMask(int cmsk, char c) {
	int nmsk = 0;
	for(int i = 0; i < len; ++i) {
		if (cmsk & (1 << i)) {
			if (pattern[i] == '?' || pattern[i] == c) {
				nmsk |= (1 << (i + 1));
			}
			if (pattern[i] == '*') {
				nmsk |= (1 << (i + 1));
				nmsk |= (1 << i);
			}
		}
	}
	return appendStars(nmsk);
}

int Automat::appendStars(int msk) const {
	for(int i = 0; i < len; ++i) {
		if (pattern[i] == '*' && (msk & (1 << i))) {
			msk |= (1 << (i + 1));
		}
	}
	return msk;
}

bool Automat::isTerminal(int s) const {
	s = states[s];
	return (s & (1 << len)) != 0;
}

void Automat::findMasks(int cmsk, set<int>& msks) {
	if (msks.count(cmsk)) {
		return ;
	}
	msks.insert(cmsk);
	for(char c = 'a'; c <= 'z'; ++c) {
		findMasks(nextMask(cmsk, c), msks);
	}
}

Automat::Automat(const string& pattern): pattern(pattern), len((int)pattern.length()) {
	set<int> msks;
	int init_msk = appendStars(1);
	findMasks(init_msk, msks);
	n = (int)msks.size();
	//cerr << n << endl;
	p.resize(n, vector<int>(26));
	states.resize(n);;
	copy(msks.begin(), msks.end(), states.begin());
	for(int i = 0; i < n; ++i) {
		int cmsk = states[i];
		for(char c = 'a'; c <= 'z'; ++c) {
			int nmsk = nextMask(cmsk, c);
			p[i][c - 'a'] = indexInSortedVector(states, nmsk);
			//cerr << p[i][c - 'a'] << " ";
		}
		//cerr << endl;
	}
	s = indexInSortedVector(states, init_msk);
	/*cerr << "----------\n" << s << endl;
	for(int i = 0; i < n; ++i) {
		cerr << isTerminal(i) << " ";
	}
	cerr << endl;*/
}

int Automat::getNumberStates() const {
	return n;
}

int Automat::goFromState(int from, char c) const {
	return p[from][c - 'a'];
}

Automat::~Automat() {
}

class Data {
public:
	Data(Automat* automat);
	Data(Automat* automat, char c);
	void append(Data& that);
	void append(char c);
	long long getNumberMatched();
private:
    int n;
	vector<long long> prefix, suffix;
    vector<int> alls;
	long long count;
	Automat* automat;
	bool empty;
};

Data::Data(Automat* automat, char c): n(automat->getNumberStates()), count(0), 
    automat(automat), empty(false), prefix(n, 0), suffix(n, 0), alls(n, 0) {
    for(int i = 0; i < n; ++i) {
        alls[i] = automat->goFromState(i, c);
    }
}

Data::Data(Automat* automat): count(0), automat(automat), empty(true) {
}

void Data::append(Data& that) {
	if (that.empty) {
		return ;
	}
	if (empty) {
		*this = that;
		return ;
	}
	//count
	count += that.count;
    for(int i = 0; i < n; ++i) {
    	count += suffix[i] * that.prefix[i];
        if (automat->isTerminal(i)) count += suffix[i];
    }
    count += that.prefix[automat->getInit()];
	//suffix
	vector<long long> nsuffix(n, 0);
    for(int i = 0; i < n; ++i) {
        nsuffix[that.alls[i]] += suffix[i]; 
    }
    nsuffix[that.alls[automat->getInit()]]++; 
    for(int i = 0; i < n; ++i) nsuffix[i] += that.suffix[i];
	suffix.swap(nsuffix);
	//prefix
    for(int i = 0; i < n; ++i) {
        prefix[i] += that.prefix[alls[i]];
        if (automat->isTerminal(alls[i])) ++prefix[i];
    }
	//alls
    for(int i = 0; i < n; ++i) alls[i] = that.alls[alls[i]];
}

void Data::append(char c) {
	Data appData(automat, c);
	append(appData);
}

long long Data::getNumberMatched() {
    if (empty) {
        return 0;
    }
	long long res = count;
	int s = automat->getInit();
	if (automat->isTerminal(alls[s])) ++res;
	for(int i = 0; i < n; ++i) {
		if (automat->isTerminal(i)) res += suffix[i];
	}
	res += prefix[s];
	return res;
}

int randomx()
{
    return ((((unsigned int)rand()) << 15) ^ ((unsigned int)rand()));
}

struct tree;
typedef tree* ptree;
struct tree {
	int x, y;
	char c;
	ptree l, r;
    Data data;
	tree();
	tree(char c);
};

Automat* automat;
ptree root, nil;

tree::tree(char c): x(1), y(randomx()), c(c), l(nil), r(nil), data(automat, c) {
}

tree::tree(): x(0), y(0), c(0), l(nil), r(nil), data(automat) {
}

void recalc(ptree v) {
	if (v != nil) {
		v->x = 1 + v->l->x + v->r->x;
		Data newData(v->l->data);
		newData.append(v->c);
		newData.append(v->r->data);
		v->data = newData;
	}
}

ptree merge(ptree l, ptree r) {
	if (l == nil) {
		return r;
	}
	if (r == nil) {
		return l;
	}
	ptree res;
	if (l->y > r->y) {
		l->r = merge(l->r, r);
		res = l;
	} else {
		r->l = merge(l, r->l);
		res = r;
	}
	recalc(res);
	return res;
}

void split(ptree v, int x, ptree& l, ptree& r) {
	if (v == nil) {
		l = r = nil;
		return ;
	}
	if (v->l->x >= x) {
		split(v->l, x, l, v->l);
		r = v;
	} else {
		split(v->r, x - v->l->x - 1, v->r, r);
		l = v;
	}
	recalc(l);
	recalc(r);
	assert(l->x == x);
}

void check(ptree v) {
	if (v != nil) {
		assert(v->y >= v->l->y);
		assert(v->y >= v->r->y);
		assert(v->x == v->l->x + v->r->x + 1);
		check(v->l);
		check(v->r);
	}
}

void add(ptree& v, int pos, char c) {
	ptree l, r;
	assert(0 <= pos && pos <= v->x);
	split(v, pos, l, r);
	ptree m = new tree(c);
	v = merge(l, m);
	v = merge(v, r);
}

void del(ptree& v, int pos) {
	ptree m, l, r;
	assert(0 <= pos && pos < v->x);
	split(v, pos, l, r);
	split(r, 1, m, r);
	v = merge(l, r);
	delete m;
}

int k;
const int MAX_BUF = 210000;
char buf[MAX_BUF + 1];

void outdata() {
}

void solve() {
	for(int i = 0; i < k; ++i) {
		scanf("%s", buf);
		char zn = buf[0];
		if (zn == '+') {
			scanf("%s", buf);
			char c = buf[0];
			int pos;
			scanf("%d", &pos);
			add(root, pos, c);
		} else {
			int pos;
			scanf("%d", &pos);
			--pos;
			del(root, pos);
		}
		cout << root->data.getNumberMatched() << endl;
		fflush(stdout);
	}
}

void delsn(string& s) {
	while (!s.empty() && (s[s.length() - 1] == 10 || s[s.length() - 1] == 13)) {
		s.erase(s.length() - 1);
	}
}

string s;

ptree createTree(int l, int r) {
    if (l > r) {
        return nil;
    }
    int m = (l + r) / 2;
    ptree tl = createTree(l, m - 1),
          tr = createTree(m + 1, r),
          tm = new tree(s[m]);
    tm->l = tl;
    tm->r = tr;
    recalc(tm);
    tm->y = tm->x;
    return tm;
}

void readdata() {
	fgets(buf, MAX_BUF, stdin);
	string pat(buf);
	delsn(pat);
	assert(1 <= pat.length() && pat.length() <= 5);
	automat = new Automat(pat);
	fgets(buf, MAX_BUF, stdin);
	s = string(buf);
	delsn(s);
	assert(1 <= s.length() && s.length() <= 100000);
	nil = new tree();
	nil->l = nil->r = nil;
	root = createTree(0, s.length() - 1);
/*	
    root = nil;
    for(int i = 0; i < (int)s.length(); ++i) {
		root = merge(root, new tree(s[i]));
	}*/
	scanf("%d", &k);
}

int main() {
//    freopen("input.txt", "rt", stdin);
//    freopen(C_OUT_FILE, "wt", stdout);
	readdata();
	solve();
	outdata();
	return 0;
}
