#include <cassert>
#include <cstdio>
#include <cstring>
#include <queue>
#include <utility>
#include <vector>

typedef std::pair<int, int> Edge;
typedef std::pair<Edge, int> MarkedEdge;

class Forest {
public:
	Forest(int numNodes);
	void augment(const std::vector<Edge> &e);
	bool checkAdd(const Edge &e);
	bool checkRemoveAdd(const Edge &e1, const Edge &e2);
private:
	int numNodes;
	std::vector<std::vector<bool> > isEdge;
	std::vector<std::vector<int> > lca;
	std::vector<int> enter;
	std::vector<int> leave;
	std::vector<int> parent;
	
	bool isParent(int u, int v);
	bool onPath(const Edge &e, int u, int v);
	void traverse(int node, int parent, std::vector<bool> &used, int &globalTime);
	int lazyLCA(int u, int v);
};

bool Forest::isParent(int u, int v) {
	return enter[u] <= enter[v] && leave[v] <= leave[u];
}

bool Forest::onPath(const Edge &e, int u, int v) {
	return isParent(u, e.first) && isParent(u, e.second) && isParent(e.first, v) && isParent(e.second, v);
}

void Forest::traverse(int node, int parent, std::vector<bool> &used, int &globalTime) {
	this->parent[node] = parent;
	enter[node] = globalTime++;
	used[node] = true;
	for (int i = 0; i < numNodes; i++) {
		if (!isEdge[node][i]) continue;
		if (used[i]) continue;
		if (i == parent) continue;
		traverse(i, node, used, globalTime);
	}
	leave[node] = globalTime++;
}

int Forest::lazyLCA(int u, int v) {
	if (lca[u][v] != -2) return lca[u][v];
	int &res = lca[u][v];
	if (isParent(u, v)) res = u;
	else if (isParent(v, u)) res = v;
	else if (parent[u] == -1) res = -1;
	else res = lazyLCA(parent[u], v);
	return res;
}

Forest::Forest(int numNodes):
	numNodes(numNodes),
	isEdge(numNodes, std::vector<bool>(numNodes, false)),
	lca(numNodes, std::vector<int>(numNodes, -1)),
	enter(numNodes),
	leave(numNodes),
	parent(numNodes, -1) {
	for (int i = 0; i < numNodes; i++) {
		lca[i][i] = i;
		enter[i] = 2 * i;
		leave[i] = 2 * i + 1;
	}
}

void Forest::augment(const std::vector<Edge> &e) {
	for (std::vector<Edge>::const_iterator it = e.begin(); it != e.end(); it++) {
		isEdge[it->first][it->second] = !isEdge[it->first][it->second];
		isEdge[it->second][it->first] = !isEdge[it->second][it->first];
	}
	std::vector<bool> used(numNodes);
	int globalTime = 0;
	for (int i = 0; i < numNodes; i++) {
		if (used[i]) continue;
		traverse(i, -1, used, globalTime);
	}
	for (int i = 0; i < numNodes; i++) {
		for (int j = 0; j < numNodes; j++) {
			lca[i][j] = -2;
		}
	}
	for (int i = 0; i < numNodes; i++) {
		for (int j = 0; j < numNodes; j++) {
			lazyLCA(i, j);
		}
	}
}

bool Forest::checkAdd(const Edge &e) {
	return lca[e.first][e.second] == -1;
}

bool Forest::checkRemoveAdd(const Edge &e1, const Edge &e2) {
	if (checkAdd(e2)) return true;
	int r = lca[e2.first][e2.second];
	return onPath(e1, r, e2.first) || onPath(e1, r, e2.second);
}

class AcyclicSubsets {
public:
	AcyclicSubsets(int numNodes);
	void augment(const std::vector<MarkedEdge> &e);
	bool checkAdd(const MarkedEdge &e);
	bool checkRemoveAdd(const MarkedEdge &e1, const MarkedEdge &e2);
private:
	Forest f1;
	Forest f2;
};

AcyclicSubsets::AcyclicSubsets(int numNodes): f1(numNodes), f2(numNodes) {
}

void AcyclicSubsets::augment(const std::vector<MarkedEdge> &e) {
	std::vector<Edge> e1, e2;
	for (std::vector<MarkedEdge>::const_iterator it = e.begin(); it != e.end(); it++) {
		if (it->second == 1) e1.push_back(it->first);
		else e2.push_back(it->first);
	}
	f1.augment(e1);
	f2.augment(e2);
}

bool AcyclicSubsets::checkAdd(const MarkedEdge &e) {
	if (e.second == 1) return f1.checkAdd(e.first);
	return f2.checkAdd(e.first);
}

bool AcyclicSubsets::checkRemoveAdd(const MarkedEdge &e1, const MarkedEdge &e2) {
	if (e1.second != e2.second) return checkAdd(e2);
	if (e1.second == 1) return f1.checkRemoveAdd(e1.first, e2.first);
	return f2.checkRemoveAdd(e1.first, e2.first);
}

class Partition {
public:
	Partition(int numEdges);
	void augment(const std::vector<int> &e);
	bool checkAdd(int e);
	bool checkRemoveAdd(int e1, int e2);
private:
	int numEdges;
	std::vector<bool> isEdge;
};

Partition::Partition(int numEdges):
	numEdges(numEdges),
	isEdge(numEdges) {
}

void Partition::augment(const std::vector<int> &e) {
	for (std::vector<int>::const_iterator it = e.begin(); it != e.end(); it++) {
		isEdge[*it] = !isEdge[*it];
	}
}

bool Partition::checkAdd(int e) {
	return !isEdge[e];
}

bool Partition::checkRemoveAdd(int e1, int e2) {
	isEdge[e1] = false;
	bool res = checkAdd(e2);
	isEdge[e1] = true;
	return res;
}

const int MAXN = 1200;
const int MAXM = 2500;

int n;
int m;
int u[MAXM];
int v[MAXM];
bool inA[MAXM][2];
bool was[MAXM][2];
std::pair<int, int> prev[MAXM][2];

int main() {
	freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
	int n, m;
	if (scanf("%d%d", &n, &m) != 2) assert(false);
	assert(1 <= n && n <= MAXN);
	assert(1 <= m && m <= MAXM);
	assert(m == 2 * (n - 1));
	for (int i = 0; i < m; i++) {
		if (scanf("%d%d", &u[i], &v[i]) != 2) assert(false);
		assert(1 <= u[i] && u[i] <= n);
		assert(1 <= v[i] && v[i] <= n);
		assert(u[i] != v[i]);
		u[i]--;
		v[i]--;
	}
	memset(inA, 0, sizeof(inA));
	AcyclicSubsets as(n);
	Partition pt(m);
	int total = 0;
	for (;;) {
		std::queue<std::pair<int, int> > q;
		memset(was, 0, sizeof(was));
		for (int i = 0; i < m; i++) {
			for (int j = 0; j < 2; j++) {
				prev[i][j] = std::make_pair(-2, -2);
			}
		}
		for (int i = 0; i < m; i++) {
			for (int j = 0; j < 2; j++) {
				if (!inA[i][j] && as.checkAdd(MarkedEdge(Edge(u[i], v[i]), j + 1))) {
					q.push(std::make_pair(i, j));
					was[i][j] = true;
					prev[i][j] = std::make_pair(-1, -1);
				}
			}
		}
		bool found = false;
		while (!q.empty()) {
			std::pair<int, int> curPair = q.front();
			q.pop();
			int edge = curPair.first;
			int mark = curPair.second;
			if (!inA[edge][mark] && pt.checkAdd(edge)) {
				found = true;
				std::vector<std::pair<int, int> > path;
				if (n > 1000) fprintf(stderr, "augmenting: ");
				while (edge != -1) {
					if (n > 1000) fprintf(stderr, "(%d, %d) ", edge, mark);
					path.push_back(std::make_pair(edge, mark));
					inA[edge][mark] = !inA[edge][mark];
					int _edge = prev[edge][mark].first;
					int _mark = prev[edge][mark].second;
					edge = _edge;
					mark = _mark;
				}
				if (n > 1000) fprintf(stderr, "\n");
				std::vector<int> v1;
				std::vector<MarkedEdge> v2;
				for (std::vector<std::pair<int, int> >::iterator it = path.begin(); it != path.end(); it++) {
					v1.push_back(it->first);
					v2.push_back(MarkedEdge(Edge(u[it->first], v[it->first]), it->second + 1));
				}
				pt.augment(v1);
				as.augment(v2);
				break;
			}
			Edge e(u[edge], v[edge]);
			MarkedEdge me(e, mark + 1);
			bool isLeft = inA[edge][mark];
			for (int i = 0; i < m; i++) {
				for (int j = 0; j < 2; j++) {
					Edge e1(u[i], v[i]);
					MarkedEdge me1(e, j + 1);
					if (inA[i][j] == isLeft) continue;
					bool isGood;
					if (isLeft) isGood = as.checkRemoveAdd(me, me1);
					else isGood = pt.checkRemoveAdd(i, edge);
					if (isGood && !was[i][j]) {
						was[i][j] = true;
						prev[i][j] = std::make_pair(edge, mark);
						q.push(std::make_pair(i, j));						
					}
				}
			}
		}
		if (!found) break;
		total++;
	}
	assert(total == 2 * (n - 1));
	for (int i = 0; i < 2; i++) {
		for (int j = 0; j < m; j++) {
			if (inA[j][i]) printf("%d ", j + 1);
		}
		printf("\n");
	}
	return 0;
}
